/*
 * Copyright (C) 2003-2012 The Music Player Daemon Project
 * http://www.musicpd.org
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#ifndef MPD_PROTOCOL_ARGPARSER_H
#define MPD_PROTOCOL_ARGPARSER_H

#include "check.h"

#include <stdbool.h>
#include <stdint.h>

struct client;

bool
check_uint32(struct client *client, uint32_t *dst, const char *s);

bool
check_int(struct client *client, int *value_r, const char *s);

bool
check_range(struct client *client, unsigned *value_r1, unsigned *value_r2,
	    const char *s);

bool
check_unsigned(struct client *client, unsigned *value_r, const char *s);

bool
check_bool(struct client *client, bool *value_r, const char *s);

bool
check_float(struct client *client, float *value_r, const char *s);

#endif
